@@ -1,29 +1,31 @@
-;;; no-littering.el --- help keeping ~/.emacs.d clean  -*- lexical-binding: t; -*-
+;;; no-littering.el --- Help keeping ~/.config/emacs clean  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2016-2022  Jonas Bernoulli
+;; Copyright (C) 2016-2023 Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Homepage: https://github.com/emacscollective/no-littering
-;; Package-Requires: ((cl-lib "0.5"))
+;; Keywords: convenience
 
-;; This file is not part of GNU Emacs.
+;; Package-Requires: ((emacs "25.1") (compat "29.1.4.1"))
 
-;; This file is free software; you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; SPDX-License-Identifier: GPL-3.0-or-later
 
+;; This file is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published
+;; by the Free Software Foundation, either version 3 of the License,
+;; or (at your option) any later version.
+;;
 ;; This file is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
-
-;; For a full copy of the GNU General Public License
-;; see https://www.gnu.org/licenses.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this file.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
-;; Help keeping ~/.emacs.d clean.
+;; Help keeping ~/.config/emacs clean.
 
 ;; The default paths used to store configuration files and persistent
 ;; data are not consistent across Emacs packages.  This isn't just a
@@ -37,7 +39,7 @@
 ;; This package sets out to fix this by changing the values of path
 ;; variables to put configuration files in `no-littering-etc-directory'
 ;; (defaulting to "etc/" under `user-emacs-directory', thus usually
-;; "~/.emacs.d/etc/") and persistent data files in
+;; "~/.config/emacs/etc/") and persistent data files in
 ;; `no-littering-var-directory' (defaulting to "var/" under
 ;; `user-emacs-directory', thus usually "~/.emacs.d/var/"), and
 ;; by using descriptive file names and subdirectories when appropriate.
@@ -49,9 +51,10 @@
 ;; coverage.  Pull requests are highly welcome (but please follow the
 ;; conventions described below and in the pull request template).
 
-;; `no-littering' cannot help with moving existing files to the new
-;; location.  You will have to move the files manually.  See issue
-;; #79 for more information.
+;; This package does not automatically migrate existing files to their
+;; new locations, but unless you want to, you also do not have to do
+;; it completely by hand.  The contributed "migrate.org" provides some
+;; guidance and tools to help with the migration.
 
 ;;;; Usage
 
@@ -93,8 +96,8 @@
 ;; 5. If applicable, the appropriate extension is added to the file name
 ;;    so that files are visited using the appropriate major-modes and
 ;;    also to provide a hint about the kind of data stored in the file.
-;;    E.g.  if a file contains an S-expression, then the suffix should be
-;;    `*.el`.
+;;    E.g.,  if a file contains an S-expression, then the suffix should
+;;    be `*.el`.
 
 ;;;;; B) File location and subdirectories
 ;;
@@ -115,11 +118,11 @@
 ;; 4. If a package provides a "framework" for other packages to use,
 ;;    then we may reuse its directories for other packages that make use
 ;;    of that framework or otherwise "extend" the "main package".
-;;    E.g. we place all `helm` related files in `helm/`.
+;;    E.g., we place all `helm` related files in `helm/`.
 ;;
 ;; 5. If a package only defines a single variable that specifies a data
 ;;    (or config) directory, then the directory name should
-;;    nevertheless be just the package name.  E.g. the path used for
+;;    nevertheless be just the package name.  E.g., the path used for
 ;;    `sx-cache-directory` from the `sx` package is `sx/cache/`, not
 ;;    `sx-cache/`.
 ;;
@@ -148,7 +151,7 @@
 ;;;;; D) Commit messages
 ;;
 ;; 1. Please theme each package using a separate commit and use commit
-;;    messages of the form "<package>: theme <variable".
+;;    messages of the form "<package>: Theme <variable".
 ;;
 ;; 2. If a package has several path variables, then you should theme them
 ;;    all in one commit.
@@ -156,10 +159,11 @@
 ;; 3. If the variable names do not fit nicely on the summary line, then
 ;;    use a message such as:
 ;;
-;;      foo: theme variables
+;;      foo: Theme variables
 ;;
 ;;      Theme `foo-config-file', `foo-cache-directory',
 ;;      and `foo-persistent-file'.
+;;
 ;; 4. When appropriate add statements like the following to the commit
 ;;    message:
 ;;
@@ -176,6 +180,7 @@
 ;;; Code:
 
 (require 'cl-lib)
+(require 'compat)
 
 (defvar no-littering-etc-directory
   (expand-file-name (convert-standard-filename "etc/") user-emacs-directory)
@@ -212,34 +217,27 @@ This variable has to be set before `no-littering' is loaded.")
     (setq abbrev-file-name                 (etc "abbrev.el"))
     (setq auto-insert-directory            (etc "auto-insert/"))
     (setq auto-save-list-file-prefix       (var "auto-save/sessions/"))
-    (setq backup-directory-alist           (list (cons "." (var "backup/"))))
     (setq bookmark-default-file            (var "bookmark-default.el"))
     (setq calc-settings-file               (etc "calc-settings.el"))
-    (eval-after-load 'desktop
-      `(make-directory ,(var "desktop/") t))
     (setq desktop-dirname                  (var "desktop/"))
     (setq desktop-path                     (list desktop-dirname))
     (setq diary-file                       (var "diary"))
     (setq ecomplete-database-file          (var "ecomplete-database.el"))
     (setq ede-project-placeholder-cache-file (var "ede-projects.el"))
-    (eval-after-load 'erc
-      `(make-directory ,(var "erc/dcc/") t))
     (setq erc-dcc-get-default-directory    (var "erc/dcc/"))
     (setq erc-log-channels-directory       (var "erc/log-channels/"))
-    (eval-after-load 'eshell
-      `(make-directory ,(etc "eshell/") t))
     (setq eshell-aliases-file              (etc "eshell/aliases"))
     (setq eshell-directory-name            (var "eshell/"))
+    (setq eshell-login-script              (etc "eshell/login"))
+    (setq eshell-rc-script                 (etc "eshell/rc"))
     (setq eudc-options-file                (etc "eudc-options.el"))
-    (eval-after-load 'eww
-      `(make-directory ,(var "eww/") t))
     (setq eww-bookmarks-directory          (var "eww/"))
     (setq filesets-menu-cache-file         (var "filesets-menu-cache.el"))
     (setq gamegrid-user-score-file-directory (var "gamegrid-user-score/"))
-    (eval-after-load 'gnus
-      `(make-directory ,(var "gnus/dribble/") t))
     (setq gnus-dribble-directory           (var "gnus/dribble/"))
     (setq gnus-init-file                   (etc "gnus/init.el"))
+    ;; Gnus hardcodes newsrc.eld to be based on gnus-startup-file.
+    (setq gnus-startup-file                (etc "gnus/newsrc"))
     (setq ido-save-directory-list-file     (var "ido-save-directory-list.el"))
     (setq image-dired-db-file              (var "image-dired/db.el"))
     (setq image-dired-dir                  (var "image-dired/"))
@@ -248,13 +246,9 @@ This variable has to be set before `no-littering' is loaded.")
     (setq image-dired-temp-rotate-image-file (var "image-dired/temp-rotate-image"))
     (setq kkc-init-file-name               (var "kkc-init.el"))
     (setq multisession-directory           (var "multisession/"))
-    (eval-after-load 'newsticker
-      `(make-directory ,(var "newsticker/") t))
     (setq newsticker-cache-filename        (var "newsticker/cache.el"))
     (setq newsticker-dir                   (var "newsticker/data/"))
     (setq nsm-settings-file                (var "nsm-settings.el"))
-    (eval-after-load 'org
-      `(make-directory ,(var "org/") t))
     (setq org-clock-persist-file           (var "org/clock-persist.el"))
     (setq org-id-locations-file            (var "org/id-locations.el"))
     (setq org-persist-directory            (var "org/persist/"))
@@ -263,17 +257,15 @@ This variable has to be set before `no-littering' is loaded.")
     (setq quickurl-url-file                (var "quickurl-url.el"))
     (setq rcirc-log-directory              (var "rcirc-log/"))
     (setq recentf-save-file                (var "recentf-save.el"))
-    (setq remember-data-file               (var "remember/data"))
     (setq remember-data-directory          (var "remember/data.d/"))
+    (setq remember-data-file               (var "remember/data"))
     (setq save-place-file                  (var "save-place.el"))
     (setq savehist-file                    (var "savehist.el"))
-    (setq srecode-map-save-file            (var "srecode-map.el"))
     (setq semanticdb-default-save-directory (var "semantic/"))
-    (eval-after-load 'shadowfile
-      `(make-directory ,(var "shadow/") t))
     (setq shadow-info-file                 (var "shadow/info.el"))
     (setq shadow-todo-file                 (var "shadow/todo.el"))
     (setq shared-game-score-directory      (var "shared-game-score/"))
+    (setq srecode-map-save-file            (var "srecode-map.el"))
     (setq timeclock-file                   (var "timeclock"))
     (setq tramp-auto-save-directory        (var "tramp/auto-save/"))
     (setq tramp-persistency-file-name      (var "tramp/persistency.el"))
@@ -283,34 +275,37 @@ This variable has to be set before `no-littering' is loaded.")
     (setq url-cookie-file                  (var "url/cookies.el"))
     (setq url-history-file                 (var "url/history.el"))
 
+    (eval-after-load 'desktop     '(make-directory desktop-dirname t))
+    (eval-after-load 'erc         '(make-directory erc-dcc-get-default-directory t))
+    (eval-after-load 'eshell      '(make-directory eshell-directory-name t))
+    (eval-after-load 'eww         '(make-directory eww-bookmarks-directory t))
+    (eval-after-load 'gnus        `(make-directory ,(etc "gnus/") t))
+    (eval-after-load 'gnus        '(make-directory gnus-dribble-directory t))
+    (eval-after-load 'newsticker  `(make-directory ,(var "newsticker/") t))
+    (eval-after-load 'org         `(make-directory ,(var "org/") t))
+    (eval-after-load 'shadowfile  `(make-directory ,(var "shadow/") t))
+
 ;;; Third-party packages
 
+    (setq abm-file                         (var "autobookmarks.el"))
     (setq ac-comphist-file                 (var "ac-comphist.el"))
     (setq amx-save-file                    (var "amx-save.el"))
     (setq anaconda-mode-installation-directory (var "anaconda-mode/"))
     (setq annotate-file                    (var "annotations.el"))
     (setq async-byte-compile-log-file      (var "async-bytecomp.log"))
     (setq auto-package-update-last-update-day-path (var "auto-package-update-last-update-day"))
-    (eval-after-load 'bbdb
-      `(make-directory ,(var "bbdb/") t))
     (setq bbdb-file                        (var "bbdb/bbdb.el"))
     (setq bbdb-vcard-directory             (var "bbdb/vcard/"))
     (setq bm-repository-file               (var "bm-repository.el"))
-    (eval-after-load 'bookmark+-1
-      `(make-directory ,(var "bmkp/") t))
     (setq bmkp-bmenu-commands-file         (var "bmkp/bmenu-commands.el"))
     (setq bmkp-current-bookmark-file       (var "bmkp/current-bookmark.el"))
     (setq bmkp-last-bookmark-file          (var "bmkp/last-bookmark.el"))
     (setq bookiez-file                     (var "bookiez"))
     (setq cider-repl-history-file          (var "cider-repl-history.el"))
+    (setq clm/logging-dir                  (var "command-log-mode/logging/"))
     (setq code-review-db-database-file     (var "code-review/database.sqlite"))
     (setq code-review-download-dir         (var "code-review/downloads/"))
     (setq code-review-log-file             (var "code-review/log"))
-    (eval-after-load 'command-log-mode
-      `(make-directory ,(var "command-log-mode-logging/") t))
-    (setq clm/logging-dir                  (var "command-log-mode/logging/"))
-    (eval-after-load 'company
-      `(make-directory ,(var "company/") t))
     (setq company-statistics-file          (var "company/statistics.el"))
     (setq company-tabnine-binaries-folder  (var "company/tabnine-binaries"))
     (setq conventional-changelog-tmp-dir   (var "conventional-changelog/"))
@@ -318,43 +313,38 @@ This variable has to be set before `no-littering' is loaded.")
     (setq dap-java-test-runner             (var "lsp-java/eclipse.jdt.ls/test-runner/junit-platform-console-standalone.jar"))
     (setq dap-utils-extension-path         (var "dap/extensions/"))
     (setq debbugs-gnu-persistency-file     (var "debbugs.el"))
+    (setq detached-db-directory            (var "detached/db/"))
+    (setq detached-session-directory       (var "detached/sessions/"))
     (setq devdocs-browser-cache-directory  (var "devdocs/browser-cache/"))
     (setq devdocs-data-dir                 (var "devdocs/data/"))
     (setq dired-recent-directories-file    (var "dired-recent-directories.el"))
     (setq dirvish-cache-dir                (var "dirvish/cache"))
     (setq elbank-data-file                 (var "elbank-data.el"))
-    (setq elmo-msgdb-directory             (var "elmo/"))
-    (setq elmo-split-log-file              (var "elmo/split-log"))
-    (eval-after-load 'elfeed
-      `(make-directory ,(var "elfeed/") t))
+    (setq elfeed-autotag-files             (list (etc "elfeed/autotags.org")))
     (setq elfeed-db-directory              (var "elfeed/db/"))
     (setq elfeed-enclosure-default-dir     (var "elfeed/enclosures/"))
     (setq elfeed-score-score-file          (etc "elfeed/score/score.el"))
+    (setq elgrep-data-file                 (var "elgrep-data.el"))
+    (setq elisp-autofmt-cache-directory    (var "elisp-autofmt/cache/"))
+    (setq elmo-msgdb-directory             (var "elmo/"))
+    (setq elmo-split-log-file              (var "elmo/split-log"))
     (setq elpher-bookmarks-file            (var "elpher-bookmarks.el"))
-    (eval-after-load 'x-win
-      (let ((session-dir (var "emacs-session/")))
-        `(progn
-           (make-directory ,session-dir t)
-           (defun emacs-session-filename (session-id)
-             "Construct a filename to save the session in based on SESSION-ID.
-This function overrides the one on `x-win' to use `no-littering'
-directories."
-             (expand-file-name session-id ,session-dir)))))
+    (setq emacs-gc-stats-file              (var "emacs-gc-stats.eld"))
+    (setq ement-sessions-file              (var "ement-sessions.el"))
     (setq emms-directory                   (var "emms/"))
-    (eval-after-load 'emojify
-      `(make-directory ,(var "emojify/") t))
     (setq emojify-emojis-dir               (var "emojify/"))
     (setq epkg-repository                  (var "epkgs/"))
     (setq equake-persistent-display-file   (var "equake-persistent-display"))
+    (setq fontaine-latest-state-file       (var "fontaine-latest-state.eld"))
     (setq forge-database-file              (var "forge/database.sqlite"))
     (setq forge-post-directory             (var "forge/posts/"))
     (setq geben-temporary-file-directory   (var "geben/"))
-    (eval-after-load 'geiser
-      `(make-directory ,(var "geiser/") t))
     (setq geiser-repl-history-filename     (var "geiser/repl-history"))
+    (setq gnus-notes-file                  (var "gnus-notes/articles.el"))
+    (setq gnus-notes-top-dir               (var "gnus-notes/"))
+    (setq grammalecte-settings-file        (etc "grammalecte-settings.el"))
     (setq hackernews-visited-links-file    (var "hackernews/visited-links.el"))
-    (eval-after-load 'helm
-      `(make-directory ,(var "helm/") t))
+    (setq harpoon-cache-file               (var "harpoon/"))
     (setq helm-adaptive-history-file       (var "helm/adaptive-history.el"))
     (setq helm-backup-path                 (var "helm/backup/"))
     (setq helm-github-stars-cache-file     (var "helm/github-stars-cache.el"))
@@ -364,46 +354,38 @@ directories."
     (setq irfc-directory                   (var "irfc/"))
     (setq irony-user-dir                   (var "irony/"))
     (setq jabber-avatar-cache-directory    (var "jabber/avatar-cache"))
-    (eval-after-load 'jabber
-      `(make-directory ,(var "jabber/avatar-cache/") t))
     (setq jabber-history-dir               (var "jabber/history"))
-    (eval-after-load 'jabber
-      `(make-directory ,(var "jabber/history/") t))
     (setq keyfreq-file                     (var "keyfreq.el"))
     (setq keyfreq-file-lock                (var "keyfreq.lock"))
     (setq libbcel-oauth-store-filename     (var "libbcel-oauth-store.el.gpg"))
     (setq litable-list-file                (var "litable-list.el"))
     (setq logview-cache-filename           (var "logview-cache"))
     (setq logview-views-file               (etc "logview-views"))
-    (eval-after-load 'lookup
-      `(make-directory ,(etc "lookup/") t))
     (setq lookup-init-directory            (etc "lookup/"))
     (setq lsp-clojure-workspace-dir        (var "lsp-clojure/workspace/"))
     (setq lsp-eslint-library-choices-file  (var "lsp/eslint-library-choices.el"))
+    (setq lsp-java-server-install-dir      (var "lsp-java/eclipse.jdt.ls/server/"))
+    (setq lsp-java-workspace-dir           (var "lsp-java/workspace/"))
+    (setq lsp-ltex-user-rules-path         (var "lsp-ltex/"))
     (setq lsp-python-ms-dir                (var "lsp-python-ms/"))
-    (eval-after-load 'lsp-mode
-      `(make-directory ,(var "lsp/") t))
     (setq lsp-server-install-dir           (var "lsp/server/"))
     (setq lsp-session-file                 (var "lsp/session.el"))
-    (setq lsp-java-workspace-dir           (var "lsp-java/workspace/"))
-    (setq lsp-java-server-install-dir      (var "lsp-java/eclipse.jdt.ls/server/"))
-    (setq magithub-dir                     (var "magithub/"))
     (setq magithub-cache-file              (var "magithub/cache.el"))
+    (setq magithub-dir                     (var "magithub/"))
     (setq mc/list-file                     (var "mc-list.el"))
     (setq meghanada-server-install-dir     (var "meghanada/"))
     (setq multi-compile-history-file       (var "multi-compile-history.el"))
     (setq nix-buffer-directory-name        (var "nix-buffer/"))
+    (setq nomad-tramp-script-directory     (var "nomad-tramp/"))
     ;; The value of this variable MUST NOT end with ".el" but the
     ;; actual file name MUST end with ".el".  Use "git blame" for
     ;; more information.
     (setq notmuch-init-file                (etc "notmuch-init"))
     (setq nov-save-place-file              (var "nov-save-place.el"))
     (setq omnisharp-cache-directory        (var "omnisharp/cache"))
-    (setq org-gcal-dir                     (var "org/gcal/"))
-    (eval-after-load 'org-caldav
-      `(make-directory ,(var "org/caldav/save") t))
     (setq org-caldav-backup-file           (var "org/caldav/backup.org"))
     (setq org-caldav-save-directory        (var "org/caldav/save"))
+    (setq org-gcal-dir                     (var "org/gcal/"))
     (setq org-journal-cache-file           (var "org/journal-cache.el"))
     (setq org-recent-headings-save-file    (var "org/recent-headings.el"))
     (setq org-registry-file                (var "org/registry.el"))
@@ -415,8 +397,6 @@ directories."
     (setq persist--directory-location      (var "persist/"))
     (setq persistent-scratch-save-file     (var "persistent-scratch.el"))
     (setq persp-save-dir                   (var "persp-mode/"))
-    (eval-after-load 'projectile
-      `(make-directory ,(var "projectile/") t))
     (setq prescient-save-file              (var "prescient-save.el"))
     (setq projectile-cache-file            (var "projectile/cache.el"))
     (setq projectile-known-projects-file   (var "projectile/known-projects.el"))
@@ -425,49 +405,142 @@ directories."
     (setq purpose-layout-dirs              (list (etc "window-purpose/layouts/")))
     (setq pyim-dcache-directory            (var "pyim/dcache/"))
     (setq quack-dir                        (var "quack/"))
-    (setq rfc-mode-directory               (var "rfc-mode/"))
+    (setq racket-repl-history-directory    (var "racket-mode/repl-history/"))
     (setq request-storage-directory        (var "request/storage/"))
+    (setq rfc-mode-directory               (var "rfc-mode/"))
     (setq rime-user-data-dir               (var "rime/"))
-    (setq rmh-elfeed-org-files             (list (var "elfeed/rmh-elfeed.org")))
+    (setq rmh-elfeed-org-files             (list (etc "elfeed/rmh-elfeed.org")))
     (setq runner-init-file                 (var "runner-init.el"))
     (setq save-kill-file-name              (var "save-kill.el"))
     (setq save-visited-files-location      (var "save-visited-files-location"))
-    (eval-after-load 'sly
-      `(make-directory ,(var "sly/") t))
+    (setq session-save-file                (var "session.el"))
+    (setq shell-maker-history-path         (var "shell-maker/"))
     (setq sly-mrepl-history-file-name      (var "sly/mrepl-history"))
     (setq smex-save-file                   (var "smex-save.el"))
     (setq speed-type-gb-dir                (var "speed-type/"))
-    (eval-after-load 'sx
-      `(make-directory ,(var "sx/cache/") t))
+    (setq spell-fu-directory               (var "spell-fu/"))
+    (setq svg-lib-icons-dir                (var "svg-lib/icons/"))
     (setq sx-cache-directory               (var "sx/cache/"))
+    (setq tabspaces-session-file           (var "tabspaces-session.eld"))
     (setq tldr-directory-path              (var "tldr/"))
     (setq transient-history-file           (var "transient/history.el"))
     (setq transient-levels-file            (etc "transient/levels.el"))
     (setq transient-values-file            (etc "transient/values.el"))
-    (setq treemacs-persist-file            (var "treemacs/persist.org"))
     (setq treemacs-last-error-persist-file (var "treemacs/persist-last-error.org"))
+    (setq treemacs-persist-file            (var "treemacs/persist.org"))
     (setq undo-fu-session-directory        (var "undo-fu-session/"))
     (setq undohist-directory               (var "undohist/"))
-    (setq undo-tree-history-directory-alist (list (cons "." (var "undo-tree-hist/"))))
+    (setq uptimes-database                 (var "uptimes.el"))
     (setq user-emacs-ensime-directory      (var "ensime/"))
     (setq vimish-fold-dir                  (var "vimish-fold/"))
-    (eval-after-load 'wl
-      `(make-directory ,(etc "wanderlust") t))
-    (setq wl-init-file                     (etc "wanderlust/init.el"))
-    (setq wl-folders-file                  (etc "wanderlust/folders"))
     (setq wl-address-file                  (etc "wanderlust/address"))
     (setq wl-alias-file                    (etc "wanderlust/alias"))
-    (setq wl-x-face-file                   (etc "wanderlust/x-face"))
+    (setq wl-folders-file                  (etc "wanderlust/folders"))
+    (setq wl-init-file                     (etc "wanderlust/init.el"))
     (setq wl-temporary-file-directory      (var "wanderlust-tmp"))
+    (setq wl-x-face-file                   (etc "wanderlust/x-face"))
     (setq x86-lookup-cache-directory       (var "x86-lookup/cache/"))
-    (eval-after-load 'xkcd
-      `(make-directory ,(var "xkcd/") t))
     (setq xkcd-cache-dir                   (var "xkcd/"))
-    (eval-after-load 'yasnippet
-      `(make-directory ,(etc "yasnippet/snippets/") t))
     (setq yas-snippet-dirs                 (list (etc "yasnippet/snippets/")))
+
+    (eval-after-load 'bbdb        `(make-directory ,(var "bbdb/") t))
+    (eval-after-load 'bookmark+-1 `(make-directory ,(var "bmkp/") t))
+    (eval-after-load 'command-log-mode '(make-directory clm/logging-dir t))
+    (eval-after-load 'company     `(make-directory ,(var "company/") t))
+    (eval-after-load 'elfeed      `(make-directory ,(var "elfeed/") t))
+    (eval-after-load 'emojify     '(make-directory emojify-emojis-dir t))
+    (eval-after-load 'geiser      `(make-directory ,(var "geiser/") t))
+    (eval-after-load 'helm        `(make-directory ,(var "helm/") t))
+    (eval-after-load 'jabber      '(make-directory jabber-avatar-cache-directory t))
+    (eval-after-load 'jabber      '(make-directory jabber-history-dir t))
+    (eval-after-load 'lookup      '(make-directory lookup-init-directory t))
+    (eval-after-load 'lsp-mode    `(make-directory ,(var "lsp/") t))
+    (eval-after-load 'org-caldav  '(make-directory org-caldav-save-directory t))
+    (eval-after-load 'projectile  `(make-directory ,(var "projectile/") t))
+    (eval-after-load 'sly         `(make-directory ,(var "sly/") t))
+    (eval-after-load 'sx          '(make-directory sx-cache-directory t))
+    (eval-after-load 'wl          `(make-directory ,(etc "wanderlust") t))
+    (eval-after-load 'xkcd        '(make-directory xkcd-cache-dir t))
+    (eval-after-load 'yasnippet   '(make-directory (car yas-snippet-dirs) t))
     ))
 
+;;; Advices
+
+(defun no-littering-emacs-session-filename (session-id)
+  "Construct a filename to save the session in, based on SESSION-ID.
+Unconditionally return a filename in `no-littering-var-directory'."
+  (let ((dir (no-littering-expand-var-file-name "emacs-session/")))
+    (make-directory dir t)
+    (expand-file-name session-id dir)))
+
+(advice-add 'emacs-session-filename :override
+            #'no-littering-emacs-session-filename)
+
+;;; Backups
+
+(defvar undo-tree-history-directory-alist)
+
+(defun no-littering-theme-backups ()
+  "Theme locations where backups of various sorts are created.
+
+The purpose of this package is to store data files of various
+sorts in a handful of central locations, instead of spreading
+them all over the place.  When doing that for temporary files,
+which contain backups of some sort, that increases the odds that
+sensitive data is written to disk in clear text and/or that such
+clear text files persist longer, if they would be created anyway.
+
+Because of that, simply loading `no-littering' does not theme
+certain, potentially unsafe variables.  Instead, this function is
+provided, so that you can decide whether to take the risk or not.
+
+Calling this function sets these variables:
+- `auto-save-file-name-transforms' (built-in)
+- `backup-directory-alist' (built-in)
+- `undo-tree-history-directory-alist' (from `undo-tree')
+
+The default values of these variables cause additional files to
+be created in the same directories as the files that are being
+visited.  Calling this function changes the values of these
+variables, so that this is only done for visited files located in
+certain directories.  For all other visited files, the additional
+files are created in files inside `no-littering-var-directory'.
+
+Additional files are created in the same directory as the visited
+file, for files located in:
+- \"/tmp/\"
+- \"/dev/shm\"
+- `temporary-file-directory'
+
+With these settings it is still possible that sensitive data is
+written to additional files, but you are more likely to spot it,
+and because these directories usually use a `tmpfs' file-system,
+the leaked secrets should not persist after a reboot.
+
+If you do *not* call this function, then these additional files
+are always created in the same directory as the visited files,
+regardless of the location of the visited files.  In other words,
+even when using the default values, there is a significant risk
+of leaking sensitive data, and if you want to reduce that, then
+you must turn of these features completely."
+  (setq auto-save-file-name-transforms
+        `(("\\`/[^/]*:\\([^/]*/\\)*\\([^/]*\\)\\'"
+           ,(concat (file-name-as-directory temporary-file-directory) "\\2") t)
+          ("\\`/tmp\\([^/]*/\\)*\\(.*\\)\\'" "\\2")
+          ("\\`/dev/shm\\([^/]*/\\)*\\(.*\\)\\'" "\\2")
+          (".*" ,(no-littering-expand-var-file-name "auto-save/") t)))
+  (setq backup-directory-alist
+        `((,(concat "\\`" (file-name-as-directory temporary-file-directory)))
+          ("\\`/tmp/" . nil)
+          ("\\`/dev/shm/" . nil)
+          ("." . ,(no-littering-expand-var-file-name "backup/"))))
+  (setq undo-tree-history-directory-alist
+        `((,(concat "\\`" (file-name-as-directory temporary-file-directory)))
+          ("\\`/tmp/" . nil)
+          ("\\`/dev/shm/" . nil)
+          ("." . ,(no-littering-expand-var-file-name "undo-tree-hist/"))))
+  )
+
 ;;; _
 (provide 'no-littering)
 ;; Local Variables:
